@@ -1,6 +1,6 @@
 from __future__ import print_function
 
-from harpoon.errors import NoSuchKey, BadOption, NoSuchImage, BadCommand, BadImage, ProgrammerError, HarpoonError, FailedImage, BadResult
+from harpoon.errors import NoSuchKey, BadOption, NoSuchImage, BadCommand, BadImage, ProgrammerError, HarpoonError, FailedImage, BadResult, UserQuit
 from harpoon.formatter import MergedOptionStringFormatter
 from harpoon.helpers import a_temp_file, until
 from harpoon.processes import command_output
@@ -11,6 +11,7 @@ from contextlib import contextmanager
 import dockerpty
 import humanize
 import fnmatch
+import hashlib
 import tarfile
 import logging
 import socket
@@ -233,7 +234,9 @@ class Image(object):
                         log.warning("Failed to stop dependency container\timage=%s\tdependency=%s\tcontainer_name=%s\terror=%s", self.name, dependency, images[dependency].container_name, error)
                 self.stop_container()
 
-    def _run_container(self, name, image_name, container_name, detach=False, command=None, tty=True, volumes=None, volumes_from=None, links=None, delete_on_exit=False, env=None, ports=None, dependency=False):
+    def _run_container(self, name, image_name, container_name
+            , detach=False, command=None, tty=True, volumes=None, volumes_from=None, links=None, delete_on_exit=False, env=None, ports=None, dependency=False, no_intervention=False
+            ):
         """Run a single container"""
         log.info("Creating container from %s\timage=%s\tcontainer_name=%s\ttty=%s", image_name, name, container_name, tty)
 
@@ -320,7 +323,7 @@ class Image(object):
                     except Exception as error:
                         log.error("Failed to see if container exited normally or not\thash=%s\terror=%s", container_id, error)
 
-            if inspection:
+            if inspection and not no_intervention:
                 if not inspection["State"]["Running"] and inspection["State"]["ExitCode"] != 0:
                     if self.interactive and not self.heira_formatted("harpoon.no_intervention", default=False):
                         print("!!!!")
@@ -503,7 +506,7 @@ class Image(object):
                                 self.docker_context.remove_image(image)
                             except Exception as error:
                                 log.error("Failed to remove replaced image\thash=%s\terror=%s", image, error)
-            except:
+            except (KeyboardInterrupt, Exception) as error:
                 exc_info = sys.exc_info()
                 if current_hash:
                     with self.intervention(current_hash):
@@ -518,7 +521,10 @@ class Image(object):
                         except Exception as error:
                             log.error("Failed to remove dead container\thash=%s\terror=%s", current_hash, error)
 
-                raise exc_info[1], None, exc_info[2]
+                if isinstance(error, KeyboardInterrupt):
+                    raise UserQuit()
+                else:
+                    raise exc_info[1], None, exc_info[2]
 
     @contextmanager
     def make_context(self):
@@ -619,19 +625,21 @@ class Image(object):
     @contextmanager
     def commit_and_run(self, container_id, command="/bin/bash"):
         """Commit this container id and run the provided command in it and clean up afterwards"""
+        image_hash = None
         try:
             image = self.docker_context.commit(container_id)
             image_hash = image["Id"]
 
             name = "{0}-intervention-{1}".format(container_id, str(uuid.uuid1()))
-            self._run_container(name, image_hash, image_hash, detach=False, tty=True, command=command, delete_on_exit=True)
+            self._run_container(name, image_hash, image_hash, detach=False, tty=True, command=command, delete_on_exit=True, no_intervention=True)
             yield
         except Exception as error:
             log.error("Something failed about creating the intervention image\terror=%s", error)
             raise
         finally:
             try:
-                self.docker_context.remove_image(image_hash)
+                if image_hash:
+                    self.docker_context.remove_image(image_hash)
             except Exception as error:
                 log.error("Failed to kill intervened image\thash=%s\terror=%s", image_hash, error)
 
@@ -786,7 +794,7 @@ class Image(object):
                 if "dest" not in value:
                     raise BadOption("When doing an ADD with content, must specify dest", image=self.name, command=[name, value])
                 dest = value.get("dest")
-                context_name = str(uuid.uuid1())
+                context_name = "{0}-{1}".format(hashlib.md5(value.get('content')).hexdigest(), dest.replace("/", "-").replace(" ", "--"))
                 self.extra_context.append((value.get("content"), context_name))
                 yield "ADD {0} {1}".format(context_name, dest)
             else:
